# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: thorchain/v1/common/common.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n thorchain/v1/common/common.proto\x12\x06\x63ommon\x1a\x14gogoproto/gogo.proto\"w\n\x05\x41sset\x12\x18\n\x05\x63hain\x18\x01 \x01(\tB\t\xfa\xde\x1f\x05\x43hain\x12\x1a\n\x06symbol\x18\x02 \x01(\tB\n\xfa\xde\x1f\x06Symbol\x12\x1a\n\x06ticker\x18\x03 \x01(\tB\n\xfa\xde\x1f\x06Ticker\x12\r\n\x05synth\x18\x04 \x01(\x08\x12\r\n\x05trade\x18\x05 \x01(\x08\"}\n\x04\x43oin\x12\"\n\x05\x61sset\x18\x01 \x01(\x0b\x32\r.common.AssetB\x04\xc8\xde\x1f\x00\x12?\n\x06\x61mount\x18\x02 \x01(\tB/\xc8\xde\x1f\x00\xda\xde\x1f\'github.com/cosmos/cosmos-sdk/types.Uint\x12\x10\n\x08\x64\x65\x63imals\x18\x03 \x01(\x03\"T\n\tPubKeySet\x12*\n\tsecp256k1\x18\x01 \x01(\tB\x17\xe2\xde\x1f\tSecp256k1\xfa\xde\x1f\x06PubKey\x12\x1b\n\x07\x65\x64\x32\x35\x35\x31\x39\x18\x02 \x01(\tB\n\xfa\xde\x1f\x06PubKey\"\xe0\x01\n\x02Tx\x12\x1a\n\x02id\x18\x01 \x01(\tB\x0e\xe2\xde\x1f\x02ID\xfa\xde\x1f\x04TxID\x12\x18\n\x05\x63hain\x18\x02 \x01(\tB\t\xfa\xde\x1f\x05\x43hain\x12!\n\x0c\x66rom_address\x18\x03 \x01(\tB\x0b\xfa\xde\x1f\x07\x41\x64\x64ress\x12\x1f\n\nto_address\x18\x04 \x01(\tB\x0b\xfa\xde\x1f\x07\x41\x64\x64ress\x12*\n\x05\x63oins\x18\x05 \x03(\x0b\x32\x0c.common.CoinB\r\xc8\xde\x1f\x00\xaa\xdf\x1f\x05\x43oins\x12&\n\x03gas\x18\x06 \x03(\x0b\x32\x0c.common.CoinB\x0b\xc8\xde\x1f\x00\xaa\xdf\x1f\x03Gas\x12\x0c\n\x04memo\x18\x07 \x01(\t\"w\n\x03\x46\x65\x65\x12*\n\x05\x63oins\x18\x01 \x03(\x0b\x32\x0c.common.CoinB\r\xc8\xde\x1f\x00\xaa\xdf\x1f\x05\x43oins\x12\x44\n\x0bpool_deduct\x18\x02 \x01(\tB/\xc8\xde\x1f\x00\xda\xde\x1f\'github.com/cosmos/cosmos-sdk/types.Uint\"Q\n\tProtoUint\x12>\n\x05value\x18\x01 \x01(\tB/\xc8\xde\x1f\x00\xda\xde\x1f\'github.com/cosmos/cosmos-sdk/types.Uint:\x04\x80\xdc \x01\x42\x32Z$gitlab.com/thorchain/thornode/common\xc8\xe1\x1e\x00\xd8\xe1\x1e\x00\x80\xe2\x1e\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'thorchain.v1.common.common_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z$gitlab.com/thorchain/thornode/common\310\341\036\000\330\341\036\000\200\342\036\000'
  _globals['_ASSET'].fields_by_name['chain']._loaded_options = None
  _globals['_ASSET'].fields_by_name['chain']._serialized_options = b'\372\336\037\005Chain'
  _globals['_ASSET'].fields_by_name['symbol']._loaded_options = None
  _globals['_ASSET'].fields_by_name['symbol']._serialized_options = b'\372\336\037\006Symbol'
  _globals['_ASSET'].fields_by_name['ticker']._loaded_options = None
  _globals['_ASSET'].fields_by_name['ticker']._serialized_options = b'\372\336\037\006Ticker'
  _globals['_COIN'].fields_by_name['asset']._loaded_options = None
  _globals['_COIN'].fields_by_name['asset']._serialized_options = b'\310\336\037\000'
  _globals['_COIN'].fields_by_name['amount']._loaded_options = None
  _globals['_COIN'].fields_by_name['amount']._serialized_options = b'\310\336\037\000\332\336\037\'github.com/cosmos/cosmos-sdk/types.Uint'
  _globals['_PUBKEYSET'].fields_by_name['secp256k1']._loaded_options = None
  _globals['_PUBKEYSET'].fields_by_name['secp256k1']._serialized_options = b'\342\336\037\tSecp256k1\372\336\037\006PubKey'
  _globals['_PUBKEYSET'].fields_by_name['ed25519']._loaded_options = None
  _globals['_PUBKEYSET'].fields_by_name['ed25519']._serialized_options = b'\372\336\037\006PubKey'
  _globals['_TX'].fields_by_name['id']._loaded_options = None
  _globals['_TX'].fields_by_name['id']._serialized_options = b'\342\336\037\002ID\372\336\037\004TxID'
  _globals['_TX'].fields_by_name['chain']._loaded_options = None
  _globals['_TX'].fields_by_name['chain']._serialized_options = b'\372\336\037\005Chain'
  _globals['_TX'].fields_by_name['from_address']._loaded_options = None
  _globals['_TX'].fields_by_name['from_address']._serialized_options = b'\372\336\037\007Address'
  _globals['_TX'].fields_by_name['to_address']._loaded_options = None
  _globals['_TX'].fields_by_name['to_address']._serialized_options = b'\372\336\037\007Address'
  _globals['_TX'].fields_by_name['coins']._loaded_options = None
  _globals['_TX'].fields_by_name['coins']._serialized_options = b'\310\336\037\000\252\337\037\005Coins'
  _globals['_TX'].fields_by_name['gas']._loaded_options = None
  _globals['_TX'].fields_by_name['gas']._serialized_options = b'\310\336\037\000\252\337\037\003Gas'
  _globals['_FEE'].fields_by_name['coins']._loaded_options = None
  _globals['_FEE'].fields_by_name['coins']._serialized_options = b'\310\336\037\000\252\337\037\005Coins'
  _globals['_FEE'].fields_by_name['pool_deduct']._loaded_options = None
  _globals['_FEE'].fields_by_name['pool_deduct']._serialized_options = b'\310\336\037\000\332\336\037\'github.com/cosmos/cosmos-sdk/types.Uint'
  _globals['_PROTOUINT'].fields_by_name['value']._loaded_options = None
  _globals['_PROTOUINT'].fields_by_name['value']._serialized_options = b'\310\336\037\000\332\336\037\'github.com/cosmos/cosmos-sdk/types.Uint'
  _globals['_PROTOUINT']._loaded_options = None
  _globals['_PROTOUINT']._serialized_options = b'\200\334 \001'
  _globals['_ASSET']._serialized_start=66
  _globals['_ASSET']._serialized_end=185
  _globals['_COIN']._serialized_start=187
  _globals['_COIN']._serialized_end=312
  _globals['_PUBKEYSET']._serialized_start=314
  _globals['_PUBKEYSET']._serialized_end=398
  _globals['_TX']._serialized_start=401
  _globals['_TX']._serialized_end=625
  _globals['_FEE']._serialized_start=627
  _globals['_FEE']._serialized_end=746
  _globals['_PROTOUINT']._serialized_start=748
  _globals['_PROTOUINT']._serialized_end=829
# @@protoc_insertion_point(module_scope)
